@@ -4,15 +4,15 @@ import (
 	"os"
 	"path/filepath"
 	"testing"
+
+	"gotest.tools/v3/assert"
 )
 
 func FuzzIsISO9660(f *testing.F) {
 	f.Fuzz(func(t *testing.T, fileContents []byte) {
 		imageFile := filepath.Join(t.TempDir(), "fuzz.iso")
 		err := os.WriteFile(imageFile, fileContents, 0o600)
-		if err != nil {
-			t.Fatal(err)
-		}
+		assert.NilError(t, err)
 		_, _ = IsISO9660(imageFile)
 	})
 }
