@@ -3,9 +3,12 @@ package limayaml
 import (
 	"bytes"
 	"crypto/sha256"
+	_ "embed"
+	"errors"
 	"fmt"
 	"net"
 	"os"
+	"os/user"
 	"path/filepath"
 	"runtime"
 	"slices"
@@ -13,16 +16,22 @@ import (
 	"strings"
 	"text/template"
 
+	"github.com/coreos/go-semver/semver"
 	"github.com/docker/go-units"
+	"github.com/goccy/go-yaml"
+	"github.com/lima-vm/lima/pkg/version"
 	"github.com/pbnjay/memory"
 	"github.com/sirupsen/logrus"
 	"golang.org/x/sys/cpu"
 
+	"github.com/lima-vm/lima/pkg/identifierutil"
+	. "github.com/lima-vm/lima/pkg/must"
 	"github.com/lima-vm/lima/pkg/networks"
 	"github.com/lima-vm/lima/pkg/osutil"
 	"github.com/lima-vm/lima/pkg/ptr"
 	"github.com/lima-vm/lima/pkg/store/dirnames"
 	"github.com/lima-vm/lima/pkg/store/filenames"
+	"github.com/lima-vm/lima/pkg/version/versionutil"
 )
 
 const (
@@ -37,7 +46,12 @@ const (
 	DefaultVirtiofsQueueSize int = 1024
 )
 
-var IPv4loopback1 = net.IPv4(127, 0, 0, 1)
+var (
+	IPv4loopback1 = net.IPv4(127, 0, 0, 1)
+
+	userHomeDir = Must(os.UserHomeDir())
+	currentUser = Must(user.Current())
+)
 
 func defaultCPUType() CPUType {
 	cpuType := map[Arch]string{
@@ -68,25 +82,20 @@ func defaultCPUType() CPUType {
 	return cpuType
 }
 
+//go:embed containerd.yaml
+var defaultContainerdYAML []byte
+
+type ContainerdYAML struct {
+	Archives []File
+}
+
 func defaultContainerdArchives() []File {
-	const nerdctlVersion = "1.7.6"
-	location := func(goos string, goarch string) string {
-		return "https://github.com/containerd/nerdctl/releases/download/v" + nerdctlVersion + "/nerdctl-full-" + nerdctlVersion + "-" + goos + "-" + goarch + ".tar.gz"
-	}
-	return []File{
-		{
-			Location: location("linux", "amd64"),
-			Arch:     X8664,
-			Digest:   "sha256:2c841e097fcfb5a1760bd354b3778cb695b44cd01f9f271c17507dc4a0b25606",
-		},
-		{
-			Location: location("linux", "arm64"),
-			Arch:     AARCH64,
-			Digest:   "sha256:77c747f09853ee3d229d77e8de0dd3c85622537d82be57433dc1fca4493bab95",
-		},
-		// No arm-v7
-		// No riscv64
+	var containerd ContainerdYAML
+	err := yaml.UnmarshalWithOptions(defaultContainerdYAML, &containerd, yaml.Strict())
+	if err != nil {
+		panic(fmt.Errorf("failed to unmarshal as YAML: %w", err))
 	}
+	return containerd.Archives
 }
 
 // FirstUsernetIndex gets the index of first usernet network under l.Network[]. Returns -1 if no usernet network found.
@@ -170,15 +179,82 @@ func defaultGuestInstallPrefix() string {
 //   - Networks are appended in d, y, o order
 //   - DNS are picked from the highest priority where DNS is not empty.
 //   - CACertificates Files and Certs are uniquely appended in d, y, o order
-func FillDefault(y, d, o *LimaYAML, filePath string) {
+func FillDefault(y, d, o *LimaYAML, filePath string, warn bool) {
 	instDir := filepath.Dir(filePath)
+
+	// existingLimaVersion can be empty if the instance was created with Lima prior to v0.20,
+	var existingLimaVersion string
+	if !isExistingInstanceDir(instDir) {
+		existingLimaVersion = version.Version
+	} else {
+		limaVersionFile := filepath.Join(instDir, filenames.LimaVersion)
+		if b, err := os.ReadFile(limaVersionFile); err == nil {
+			existingLimaVersion = strings.TrimSpace(string(b))
+		} else if !errors.Is(err, os.ErrNotExist) {
+			logrus.WithError(err).Warnf("Failed to read %q", limaVersionFile)
+		}
+	}
+
+	if y.User.Name == nil {
+		y.User.Name = d.User.Name
+	}
+	if y.User.Comment == nil {
+		y.User.Comment = d.User.Comment
+	}
+	if y.User.Home == nil {
+		y.User.Home = d.User.Home
+	}
+	if y.User.UID == nil {
+		y.User.UID = d.User.UID
+	}
+	if o.User.Name != nil {
+		y.User.Name = o.User.Name
+	}
+	if o.User.Comment != nil {
+		y.User.Comment = o.User.Comment
+	}
+	if o.User.Home != nil {
+		y.User.Home = o.User.Home
+	}
+	if o.User.UID != nil {
+		y.User.UID = o.User.UID
+	}
+	if y.User.Name == nil {
+		y.User.Name = ptr.Of(osutil.LimaUser(existingLimaVersion, warn).Username)
+		warn = false
+	}
+	if y.User.Comment == nil {
+		y.User.Comment = ptr.Of(osutil.LimaUser(existingLimaVersion, warn).Name)
+		warn = false
+	}
+	if y.User.Home == nil {
+		y.User.Home = ptr.Of(osutil.LimaUser(existingLimaVersion, warn).HomeDir)
+		warn = false
+	}
+	if y.User.UID == nil {
+		uidString := osutil.LimaUser(existingLimaVersion, warn).Uid
+		if uid, err := strconv.ParseUint(uidString, 10, 32); err == nil {
+			y.User.UID = ptr.Of(uint32(uid))
+		} else {
+			// This should never happen; LimaUser() makes sure that .Uid is numeric
+			logrus.WithError(err).Warnf("Can't parse `user.uid` %q", uidString)
+			y.User.UID = ptr.Of(uint32(1000))
+		}
+		// warn = false
+	}
+	if out, err := executeGuestTemplate(*y.User.Home, instDir, y.User, y.Param); err == nil {
+		y.User.Home = ptr.Of(out.String())
+	} else {
+		logrus.WithError(err).Warnf("Couldn't process `user.home` value %q as a template", *y.User.Home)
+	}
+
 	if y.VMType == nil {
 		y.VMType = d.VMType
 	}
 	if o.VMType != nil {
 		y.VMType = o.VMType
 	}
-	y.VMType = ptr.Of(ResolveVMType(y.VMType))
+	y.VMType = ptr.Of(ResolveVMType(y, d, o, filePath))
 	if y.OS == nil {
 		y.OS = d.OS
 	}
@@ -339,7 +415,7 @@ func FillDefault(y, d, o *LimaYAML, filePath string) {
 		y.SSH.LoadDotSSHPubKeys = o.SSH.LoadDotSSHPubKeys
 	}
 	if y.SSH.LoadDotSSHPubKeys == nil {
-		y.SSH.LoadDotSSHPubKeys = ptr.Of(true)
+		y.SSH.LoadDotSSHPubKeys = ptr.Of(false) // was true before Lima v1.0
 	}
 
 	if y.SSH.ForwardAgent == nil {
@@ -394,7 +470,7 @@ func FillDefault(y, d, o *LimaYAML, filePath string) {
 		if provision.Mode == ProvisionModeDependency && provision.SkipDefaultDependencyResolution == nil {
 			provision.SkipDefaultDependencyResolution = ptr.Of(false)
 		}
-		if out, err := executeGuestTemplate(provision.Script, instDir, y.Param); err == nil {
+		if out, err := executeGuestTemplate(provision.Script, instDir, y.User, y.Param); err == nil {
 			provision.Script = out.String()
 		} else {
 			logrus.WithError(err).Warnf("Couldn't process provisioning script %q as a template", provision.Script)
@@ -437,7 +513,12 @@ func FillDefault(y, d, o *LimaYAML, filePath string) {
 		y.Containerd.User = o.Containerd.User
 	}
 	if y.Containerd.User == nil {
-		y.Containerd.User = ptr.Of(true)
+		switch *y.Arch {
+		case X8664, AARCH64:
+			y.Containerd.User = ptr.Of(true)
+		default:
+			y.Containerd.User = ptr.Of(false)
+		}
 	}
 
 	y.Containerd.Archives = append(append(o.Containerd.Archives, y.Containerd.Archives...), d.Containerd.Archives...)
@@ -460,7 +541,7 @@ func FillDefault(y, d, o *LimaYAML, filePath string) {
 		if probe.Description == "" {
 			probe.Description = fmt.Sprintf("user probe %d/%d", i+1, len(y.Probes))
 		}
-		if out, err := executeGuestTemplate(probe.Script, instDir, y.Param); err == nil {
+		if out, err := executeGuestTemplate(probe.Script, instDir, y.User, y.Param); err == nil {
 			probe.Script = out.String()
 		} else {
 			logrus.WithError(err).Warnf("Couldn't process probing script %q as a template", probe.Script)
@@ -469,13 +550,13 @@ func FillDefault(y, d, o *LimaYAML, filePath string) {
 
 	y.PortForwards = append(append(o.PortForwards, y.PortForwards...), d.PortForwards...)
 	for i := range y.PortForwards {
-		FillPortForwardDefaults(&y.PortForwards[i], instDir, y.Param)
+		FillPortForwardDefaults(&y.PortForwards[i], instDir, y.User, y.Param)
 		// After defaults processing the singular HostPort and GuestPort values should not be used again.
 	}
 
 	y.CopyToHost = append(append(o.CopyToHost, y.CopyToHost...), d.CopyToHost...)
 	for i := range y.CopyToHost {
-		FillCopyToHostDefaults(&y.CopyToHost[i], instDir, y.Param)
+		FillCopyToHostDefaults(&y.CopyToHost[i], instDir, y.User, y.Param)
 	}
 
 	if y.HostResolver.Enabled == nil {
@@ -528,6 +609,9 @@ func FillDefault(y, d, o *LimaYAML, filePath string) {
 			if nw.MACAddress != "" {
 				networks[i].MACAddress = nw.MACAddress
 			}
+			if nw.Metric != nil {
+				networks[i].Metric = nw.Metric
+			}
 		} else {
 			// unnamed network definitions are not combined/overwritten
 			if nw.Interface != "" {
@@ -546,8 +630,16 @@ func FillDefault(y, d, o *LimaYAML, filePath string) {
 		if nw.Interface == "" {
 			nw.Interface = "lima" + strconv.Itoa(i)
 		}
+		if nw.Metric == nil {
+			nw.Metric = ptr.Of(uint32(100))
+		}
 	}
 
+	y.MountTypesUnsupported = append(append(o.MountTypesUnsupported, y.MountTypesUnsupported...), d.MountTypesUnsupported...)
+	mountTypesUnsupported := make(map[string]struct{})
+	for _, f := range y.MountTypesUnsupported {
+		mountTypesUnsupported[f] = struct{}{}
+	}
 	// MountType has to be resolved before resolving Mounts
 	if y.MountType == nil {
 		y.MountType = d.MountType
@@ -555,14 +647,29 @@ func FillDefault(y, d, o *LimaYAML, filePath string) {
 	if o.MountType != nil {
 		y.MountType = o.MountType
 	}
-	if y.MountType == nil || *y.MountType == "" {
-		if *y.VMType == VZ {
+	if y.MountType == nil || *y.MountType == "" || *y.MountType == "default" {
+		switch *y.VMType {
+		case VZ:
 			y.MountType = ptr.Of(VIRTIOFS)
-		} else {
+		case QEMU:
+			y.MountType = ptr.Of(NINEP)
+			if _, ok := mountTypesUnsupported[NINEP]; ok {
+				// Use REVSSHFS if the instance does not support 9p
+				y.MountType = ptr.Of(REVSSHFS)
+			} else if isExistingInstanceDir(instDir) && !versionutil.GreaterEqual(existingLimaVersion, "1.0.0") {
+				// Use REVSSHFS if the instance was created with Lima prior to v1.0
+				y.MountType = ptr.Of(REVSSHFS)
+			}
+		default:
 			y.MountType = ptr.Of(REVSSHFS)
 		}
 	}
 
+	if _, ok := mountTypesUnsupported[*y.MountType]; ok {
+		// We cannot return an error here, but Validate() will return it.
+		logrus.Warnf("Unsupported mount type: %q", *y.MountType)
+	}
+
 	if y.MountInotify == nil {
 		y.MountInotify = d.MountInotify
 	}
@@ -578,6 +685,18 @@ func FillDefault(y, d, o *LimaYAML, filePath string) {
 	mounts := make([]Mount, 0, len(d.Mounts)+len(y.Mounts)+len(o.Mounts))
 	location := make(map[string]int)
 	for _, mount := range append(append(d.Mounts, y.Mounts...), o.Mounts...) {
+		if out, err := executeHostTemplate(mount.Location, instDir, y.Param); err == nil {
+			mount.Location = out.String()
+		} else {
+			logrus.WithError(err).Warnf("Couldn't process mount location %q as a template", mount.Location)
+		}
+		if mount.MountPoint != nil {
+			if out, err := executeGuestTemplate(*mount.MountPoint, instDir, y.User, y.Param); err == nil {
+				mount.MountPoint = ptr.Of(out.String())
+			} else {
+				logrus.WithError(err).Warnf("Couldn't process mount point %q as a template", *mount.MountPoint)
+			}
+		}
 		if i, ok := location[mount.Location]; ok {
 			if mount.SSHFS.Cache != nil {
 				mounts[i].SSHFS.Cache = mount.SSHFS.Cache
@@ -606,7 +725,7 @@ func FillDefault(y, d, o *LimaYAML, filePath string) {
 			if mount.Writable != nil {
 				mounts[i].Writable = mount.Writable
 			}
-			if mount.MountPoint != "" {
+			if mount.MountPoint != nil {
 				mounts[i].MountPoint = mount.MountPoint
 			}
 		} else {
@@ -649,8 +768,8 @@ func FillDefault(y, d, o *LimaYAML, filePath string) {
 				mounts[i].NineP.Cache = ptr.Of(Default9pCacheForRO)
 			}
 		}
-		if mount.MountPoint == "" {
-			mounts[i].MountPoint = mount.Location
+		if mount.MountPoint == nil {
+			mounts[i].MountPoint = ptr.Of(mount.Location)
 		}
 	}
 
@@ -726,6 +845,16 @@ func FillDefault(y, d, o *LimaYAML, filePath string) {
 		y.Rosetta.BinFmt = ptr.Of(false)
 	}
 
+	if y.NestedVirtualization == nil {
+		y.NestedVirtualization = d.NestedVirtualization
+	}
+	if o.NestedVirtualization != nil {
+		y.NestedVirtualization = o.NestedVirtualization
+	}
+	if y.NestedVirtualization == nil {
+		y.NestedVirtualization = ptr.Of(false)
+	}
+
 	if y.Plain == nil {
 		y.Plain = d.Plain
 	}
@@ -752,16 +881,17 @@ func fixUpForPlainMode(y *LimaYAML) {
 	y.TimeZone = ptr.Of("")
 }
 
-func executeGuestTemplate(format, instDir string, param map[string]string) (bytes.Buffer, error) {
+func executeGuestTemplate(format, instDir string, user User, param map[string]string) (bytes.Buffer, error) {
 	tmpl, err := template.New("").Parse(format)
 	if err == nil {
-		user, _ := osutil.LimaUser(false)
+		name := filepath.Base(instDir)
 		data := map[string]interface{}{
-			"Home":  fmt.Sprintf("/home/%s.linux", user.Username),
-			"Name":  filepath.Base(instDir),
-			"UID":   user.Uid,
-			"User":  user.Username,
-			"Param": param,
+			"Name":     name,
+			"Hostname": identifierutil.HostnameFromInstName(name), // TODO: support customization
+			"UID":      *user.UID,
+			"User":     *user.Name,
+			"Home":     *user.Home,
+			"Param":    param,
 		}
 		var out bytes.Buffer
 		if err := tmpl.Execute(&out, data); err == nil {
@@ -774,15 +904,14 @@ func executeGuestTemplate(format, instDir string, param map[string]string) (byte
 func executeHostTemplate(format, instDir string, param map[string]string) (bytes.Buffer, error) {
 	tmpl, err := template.New("").Parse(format)
 	if err == nil {
-		user, _ := osutil.LimaUser(false)
-		home, _ := os.UserHomeDir()
 		limaHome, _ := dirnames.LimaDir()
 		data := map[string]interface{}{
-			"Dir":   instDir,
-			"Home":  home,
-			"Name":  filepath.Base(instDir),
-			"UID":   user.Uid,
-			"User":  user.Username,
+			"Dir":  instDir,
+			"Name": filepath.Base(instDir),
+			// TODO: add hostname fields for the host and the guest
+			"UID":   currentUser.Uid,
+			"User":  currentUser.Username,
+			"Home":  userHomeDir,
 			"Param": param,
 
 			"Instance": filepath.Base(instDir), // DEPRECATED, use `{{.Name}}`
@@ -796,9 +925,9 @@ func executeHostTemplate(format, instDir string, param map[string]string) (bytes
 	return bytes.Buffer{}, err
 }
 
-func FillPortForwardDefaults(rule *PortForward, instDir string, param map[string]string) {
+func FillPortForwardDefaults(rule *PortForward, instDir string, user User, param map[string]string) {
 	if rule.Proto == "" {
-		rule.Proto = TCP
+		rule.Proto = ProtoTCP
 	}
 	if rule.GuestIP == nil {
 		if rule.GuestIPMustBeZero {
@@ -828,7 +957,7 @@ func FillPortForwardDefaults(rule *PortForward, instDir string, param map[string
 		}
 	}
 	if rule.GuestSocket != "" {
-		if out, err := executeGuestTemplate(rule.GuestSocket, instDir, param); err == nil {
+		if out, err := executeGuestTemplate(rule.GuestSocket, instDir, user, param); err == nil {
 			rule.GuestSocket = out.String()
 		} else {
 			logrus.WithError(err).Warnf("Couldn't process guestSocket %q as a template", rule.GuestSocket)
@@ -846,9 +975,9 @@ func FillPortForwardDefaults(rule *PortForward, instDir string, param map[string
 	}
 }
 
-func FillCopyToHostDefaults(rule *CopyToHost, instDir string, param map[string]string) {
+func FillCopyToHostDefaults(rule *CopyToHost, instDir string, user User, param map[string]string) {
 	if rule.GuestFile != "" {
-		if out, err := executeGuestTemplate(rule.GuestFile, instDir, param); err == nil {
+		if out, err := executeGuestTemplate(rule.GuestFile, instDir, user, param); err == nil {
 			rule.GuestFile = out.String()
 		} else {
 			logrus.WithError(err).Warnf("Couldn't process guest %q as a template", rule.GuestFile)
@@ -924,11 +1053,100 @@ func NewVMType(driver string) VMType {
 	}
 }
 
-func ResolveVMType(s *string) VMType {
-	if s == nil || *s == "" || *s == "default" {
+func isExistingInstanceDir(dir string) bool {
+	// existence of "lima.yaml" does not signify existence of the instance,
+	// because the file is created during the initialization of the instance.
+	for _, f := range []string{
+		filenames.HostAgentStdoutLog, filenames.HostAgentStderrLog,
+		filenames.VzIdentifier, filenames.BaseDisk, filenames.DiffDisk, filenames.CIDataISO,
+	} {
+		file := filepath.Join(dir, f)
+		if _, err := os.Lstat(file); !errors.Is(err, os.ErrNotExist) {
+			return true
+		}
+	}
+	return false
+}
+
+func ResolveVMType(y, d, o *LimaYAML, filePath string) VMType {
+	// Check if the VMType is explicitly specified
+	for i, f := range []*LimaYAML{o, y, d} {
+		if f.VMType != nil && *f.VMType != "" && *f.VMType != "default" {
+			logrus.Debugf("ResolveVMType: resolved VMType %q (explicitly specified in []*LimaYAML{o,y,d}[%d])", *f.VMType, i)
+			return NewVMType(*f.VMType)
+		}
+	}
+
+	// If this is an existing instance, guess the VMType from the contents of the instance directory.
+	if dir, basename := filepath.Split(filePath); dir != "" && basename == filenames.LimaYAML && isExistingInstanceDir(dir) {
+		if runtime.GOOS == "darwin" {
+			vzIdentifier := filepath.Join(dir, filenames.VzIdentifier) // since Lima v0.14
+			if _, err := os.Lstat(vzIdentifier); !errors.Is(err, os.ErrNotExist) {
+				logrus.Debugf("ResolveVMType: resolved VMType %q (existing instance, with %q)", VZ, vzIdentifier)
+				return VZ
+			}
+			logrus.Debugf("ResolveVMType: resolved VMType %q (existing instance, without %q)", QEMU, vzIdentifier)
+			return QEMU
+		}
+		logrus.Debugf("ResolveVMType: resolved VMType %q (existing instance)", QEMU)
+		return QEMU
+	}
+
+	// Resolve the best type, depending on GOOS
+	switch runtime.GOOS {
+	case "darwin":
+		macOSProductVersion, err := osutil.ProductVersion()
+		if err != nil {
+			logrus.WithError(err).Warn("Failed to get macOS product version")
+			logrus.Debugf("ResolveVMType: resolved VMType %q (default for unknown version of macOS)", QEMU)
+			return QEMU
+		}
+		// Virtualization.framework in macOS prior to 13.5 could not boot Linux kernel v6.2 on Intel
+		// https://github.com/lima-vm/lima/issues/1577
+		if macOSProductVersion.LessThan(*semver.New("13.5.0")) {
+			logrus.Debugf("ResolveVMType: resolved VMType %q (default for macOS prior to 13.5)", QEMU)
+			return QEMU
+		}
+		// Use QEMU if the config depends on QEMU
+		for i, f := range []*LimaYAML{o, y, d} {
+			if f.Arch != nil && !IsNativeArch(*f.Arch) {
+				logrus.Debugf("ResolveVMType: resolved VMType %q (non-native arch=%q is specified in []*LimaYAML{o,y,d}[%d])", QEMU, *f.Arch, i)
+				return QEMU
+			}
+			if f.Firmware.LegacyBIOS != nil && *f.Firmware.LegacyBIOS {
+				logrus.Debugf("ResolveVMType: resolved VMType %q (firmware.legacyBIOS is specified in []*LimaYAML{o,y,d}[%d])", QEMU, i)
+				return QEMU
+			}
+			if f.MountType != nil && *f.MountType == NINEP {
+				logrus.Debugf("ResolveVMType: resolved VMType %q (mountType=%q is specified in []*LimaYAML{o,y,d}[%d])", QEMU, NINEP, i)
+				return QEMU
+			}
+			if f.Audio.Device != nil {
+				switch *f.Audio.Device {
+				case "", "none", "default", "vz":
+					// NOP
+				default:
+					logrus.Debugf("ResolveVMType: resolved VMType %q (audio.device=%q is specified in []*LimaYAML{o,y,d}[%d])", QEMU, *f.Audio.Device, i)
+					return QEMU
+				}
+			}
+			if f.Video.Display != nil {
+				switch *f.Video.Display {
+				case "", "none", "default", "vz":
+					// NOP
+				default:
+					logrus.Debugf("ResolveVMType: resolved VMType %q (video.display=%q is specified in []*LimaYAML{o,y,d}[%d])", QEMU, *f.Video.Display, i)
+					return QEMU
+				}
+			}
+		}
+		// Use VZ if the config is compatible with VZ
+		logrus.Debugf("ResolveVMType: resolved VMType %q (default for macOS 13.5 and later)", VZ)
+		return VZ
+	default:
+		logrus.Debugf("ResolveVMType: resolved VMType %q (default for GOOS=%q)", QEMU, runtime.GOOS)
 		return QEMU
 	}
-	return NewVMType(*s)
 }
 
 func ResolveOS(s *string) OS {
