@@ -6,6 +6,8 @@ import (
 	"os"
 	"path/filepath"
 	"runtime"
+	"slices"
+	"strconv"
 	"testing"
 
 	"github.com/google/go-cmp/cmp"
@@ -14,12 +16,16 @@ import (
 	"github.com/lima-vm/lima/pkg/ptr"
 	"github.com/lima-vm/lima/pkg/store/dirnames"
 	"github.com/lima-vm/lima/pkg/store/filenames"
+	"github.com/sirupsen/logrus"
 	"gotest.tools/v3/assert"
 )
 
 func TestFillDefault(t *testing.T) {
+	logrus.SetLevel(logrus.DebugLevel)
 	var d, y, o LimaYAML
 
+	defaultVMType := ResolveVMType(&y, &d, &o, "")
+
 	opts := []cmp.Option{
 		// Consider nil slices and empty slices to be identical
 		cmpopts.EquateEmpty(),
@@ -49,17 +55,21 @@ func TestFillDefault(t *testing.T) {
 	assert.NilError(t, err)
 	limaHome, err := dirnames.LimaDir()
 	assert.NilError(t, err)
-	user, err := osutil.LimaUser(false)
+	user := osutil.LimaUser("0.0.0", false)
+	if runtime.GOOS != "windows" {
+		// manual template expansion for "/home/{{.User}}.linux" (done by FillDefault)
+		user.HomeDir = fmt.Sprintf("/home/%s.linux", user.Username)
+	}
+	uid, err := strconv.ParseUint(user.Uid, 10, 32)
 	assert.NilError(t, err)
 
-	guestHome := fmt.Sprintf("/home/%s.linux", user.Username)
 	instName := "instance"
 	instDir := filepath.Join(limaHome, instName)
 	filePath := filepath.Join(instDir, filenames.LimaYAML)
 
 	// Builtin default values
 	builtin := LimaYAML{
-		VMType:             ptr.Of("qemu"),
+		VMType:             &defaultVMType,
 		OS:                 ptr.Of(LINUX),
 		Arch:               ptr.Of(arch),
 		CPUType:            defaultCPUType(),
@@ -75,7 +85,7 @@ func TestFillDefault(t *testing.T) {
 		},
 		SSH: SSH{
 			LocalPort:         ptr.Of(0),
-			LoadDotSSHPubKeys: ptr.Of(true),
+			LoadDotSSHPubKeys: ptr.Of(false),
 			ForwardAgent:      ptr.Of(false),
 			ForwardX11:        ptr.Of(false),
 			ForwardX11Trusted: ptr.Of(false),
@@ -101,7 +111,14 @@ func TestFillDefault(t *testing.T) {
 		CACertificates: CACertificates{
 			RemoveDefaults: ptr.Of(false),
 		},
-		Plain: ptr.Of(false),
+		NestedVirtualization: ptr.Of(false),
+		Plain:                ptr.Of(false),
+		User: User{
+			Name:    ptr.Of(user.Username),
+			Comment: ptr.Of(user.Name),
+			Home:    ptr.Of(user.HomeDir),
+			UID:     ptr.Of(uint32(uid)),
+		},
 	}
 
 	defaultPortForward := PortForward{
@@ -109,7 +126,7 @@ func TestFillDefault(t *testing.T) {
 		GuestPortRange: [2]int{1, 65535},
 		HostIP:         IPv4loopback1,
 		HostPortRange:  [2]int{1, 65535},
-		Proto:          TCP,
+		Proto:          ProtoTCP,
 		Reverse:        false,
 	}
 
@@ -126,6 +143,7 @@ func TestFillDefault(t *testing.T) {
 		},
 		Mounts: []Mount{
 			{Location: "/tmp"},
+			{Location: "{{.Dir}}/{{.Param.ONE}}", MountPoint: ptr.Of("/mnt/{{.Param.ONE}}")},
 		},
 		MountType: ptr.Of(NINEP),
 		Provision: []Provision{
@@ -192,12 +210,13 @@ func TestFillDefault(t *testing.T) {
 	}
 
 	expect := builtin
+	expect.VMType = ptr.Of(QEMU) // due to NINEP
 	expect.HostResolver.Hosts = map[string]string{
 		"MY.Host": "host.lima.internal",
 	}
 
-	expect.Mounts = y.Mounts
-	expect.Mounts[0].MountPoint = expect.Mounts[0].Location
+	expect.Mounts = slices.Clone(y.Mounts)
+	expect.Mounts[0].MountPoint = ptr.Of(expect.Mounts[0].Location)
 	expect.Mounts[0].Writable = ptr.Of(false)
 	expect.Mounts[0].SSHFS.Cache = ptr.Of(true)
 	expect.Mounts[0].SSHFS.FollowSymlinks = ptr.Of(false)
@@ -206,27 +225,39 @@ func TestFillDefault(t *testing.T) {
 	expect.Mounts[0].NineP.ProtocolVersion = ptr.Of(Default9pProtocolVersion)
 	expect.Mounts[0].NineP.Msize = ptr.Of(Default9pMsize)
 	expect.Mounts[0].NineP.Cache = ptr.Of(Default9pCacheForRO)
-	expect.Mounts[0].Virtiofs.QueueSize = ptr.Of(DefaultVirtiofsQueueSize)
+	expect.Mounts[0].Virtiofs.QueueSize = nil
 	// Only missing Mounts field is Writable, and the default value is also the null value: false
+	expect.Mounts[1].Location = fmt.Sprintf("%s/%s", instDir, y.Param["ONE"])
+	expect.Mounts[1].MountPoint = ptr.Of(fmt.Sprintf("/mnt/%s", y.Param["ONE"]))
+	expect.Mounts[1].Writable = ptr.Of(false)
+	expect.Mounts[1].SSHFS.Cache = ptr.Of(true)
+	expect.Mounts[1].SSHFS.FollowSymlinks = ptr.Of(false)
+	expect.Mounts[1].SSHFS.SFTPDriver = ptr.Of("")
+	expect.Mounts[1].NineP.SecurityModel = ptr.Of(Default9pSecurityModel)
+	expect.Mounts[1].NineP.ProtocolVersion = ptr.Of(Default9pProtocolVersion)
+	expect.Mounts[1].NineP.Msize = ptr.Of(Default9pMsize)
+	expect.Mounts[1].NineP.Cache = ptr.Of(Default9pCacheForRO)
+	expect.Mounts[1].Virtiofs.QueueSize = nil
 
 	expect.MountType = ptr.Of(NINEP)
 
 	expect.MountInotify = ptr.Of(false)
 
-	expect.Provision = y.Provision
+	expect.Provision = slices.Clone(y.Provision)
 	expect.Provision[0].Mode = ProvisionModeSystem
 	expect.Provision[0].Script = "#!/bin/true # Eins"
 
-	expect.Probes = y.Probes
+	expect.Probes = slices.Clone(y.Probes)
 	expect.Probes[0].Mode = ProbeModeReadiness
 	expect.Probes[0].Description = "user probe 1/1"
-	expect.Probes[0].Script = "#!/bin/true # Eins"
+	expect.Probes[0].Script = "#!/bin/false # Eins"
 
-	expect.Networks = y.Networks
+	expect.Networks = slices.Clone(y.Networks)
 	expect.Networks[0].MACAddress = MACAddress(fmt.Sprintf("%s#%d", filePath, 0))
 	expect.Networks[0].Interface = "lima0"
+	expect.Networks[0].Metric = ptr.Of(uint32(100))
 
-	expect.DNS = y.DNS
+	expect.DNS = slices.Clone(y.DNS)
 	expect.PortForwards = []PortForward{
 		defaultPortForward,
 		defaultPortForward,
@@ -248,11 +279,11 @@ func TestFillDefault(t *testing.T) {
 	expect.PortForwards[2].HostPort = 8888
 	expect.PortForwards[2].HostPortRange = [2]int{8888, 8888}
 
-	expect.PortForwards[3].GuestSocket = fmt.Sprintf("%s | %s | %s | %s", guestHome, user.Uid, user.Username, y.Param["ONE"])
-	expect.PortForwards[3].HostSocket = fmt.Sprintf("%s | %s | %s | %s | %s | %s", hostHome, instDir, instName, user.Uid, user.Username, y.Param["ONE"])
+	expect.PortForwards[3].GuestSocket = fmt.Sprintf("%s | %s | %s | %s", user.HomeDir, user.Uid, user.Username, y.Param["ONE"])
+	expect.PortForwards[3].HostSocket = fmt.Sprintf("%s | %s | %s | %s | %s | %s", hostHome, instDir, instName, currentUser.Uid, currentUser.Username, y.Param["ONE"])
 
-	expect.CopyToHost[0].GuestFile = fmt.Sprintf("%s | %s | %s | %s", guestHome, user.Uid, user.Username, y.Param["ONE"])
-	expect.CopyToHost[0].HostFile = fmt.Sprintf("%s | %s | %s | %s | %s | %s", hostHome, instDir, instName, user.Uid, user.Username, y.Param["ONE"])
+	expect.CopyToHost[0].GuestFile = fmt.Sprintf("%s | %s | %s | %s", user.HomeDir, user.Uid, user.Username, y.Param["ONE"])
+	expect.CopyToHost[0].HostFile = fmt.Sprintf("%s | %s | %s | %s | %s | %s", hostHome, instDir, instName, currentUser.Uid, currentUser.Username, y.Param["ONE"])
 
 	expect.Env = y.Env
 
@@ -268,13 +299,16 @@ func TestFillDefault(t *testing.T) {
 
 	expect.TimeZone = y.TimeZone
 	expect.Firmware = y.Firmware
+	expect.Firmware.Images = slices.Clone(y.Firmware.Images)
 
 	expect.Rosetta = Rosetta{
 		Enabled: ptr.Of(false),
 		BinFmt:  ptr.Of(false),
 	}
 
-	FillDefault(&y, &LimaYAML{}, &LimaYAML{}, filePath)
+	expect.NestedVirtualization = ptr.Of(false)
+
+	FillDefault(&y, &LimaYAML{}, &LimaYAML{}, filePath, false)
 	assert.DeepEqual(t, &y, &expect, opts...)
 
 	filledDefaults := y
@@ -368,6 +402,7 @@ func TestFillDefault(t *testing.T) {
 			{
 				MACAddress: "11:22:33:44:55:66",
 				Interface:  "def0",
+				Metric:     ptr.Of(uint32(50)),
 			},
 		},
 		DNS: []net.IP{
@@ -380,7 +415,7 @@ func TestFillDefault(t *testing.T) {
 			HostIP:         IPv4loopback1,
 			HostPort:       80,
 			HostPortRange:  [2]int{80, 80},
-			Proto:          TCP,
+			Proto:          ProtoTCP,
 		}},
 		CopyToHost: []CopyToHost{{}},
 		Env: map[string]string{
@@ -401,12 +436,21 @@ func TestFillDefault(t *testing.T) {
 			Enabled: ptr.Of(true),
 			BinFmt:  ptr.Of(true),
 		},
+		NestedVirtualization: ptr.Of(true),
+		User: User{
+			Name:    ptr.Of("xxx"),
+			Comment: ptr.Of("Foo Bar"),
+			Home:    ptr.Of("/tmp"),
+			UID:     ptr.Of(uint32(8080)),
+		},
 	}
 
 	expect = d
 	// Also verify that archive arch is filled in
+	expect.Containerd.Archives = slices.Clone(d.Containerd.Archives)
 	expect.Containerd.Archives[0].Arch = *d.Arch
-	expect.Mounts[0].MountPoint = expect.Mounts[0].Location
+	expect.Mounts = slices.Clone(d.Mounts)
+	expect.Mounts[0].MountPoint = ptr.Of(expect.Mounts[0].Location)
 	expect.Mounts[0].SSHFS.Cache = ptr.Of(true)
 	expect.Mounts[0].SSHFS.FollowSymlinks = ptr.Of(false)
 	expect.Mounts[0].SSHFS.SFTPDriver = ptr.Of("")
@@ -414,7 +458,7 @@ func TestFillDefault(t *testing.T) {
 	expect.Mounts[0].NineP.ProtocolVersion = ptr.Of(Default9pProtocolVersion)
 	expect.Mounts[0].NineP.Msize = ptr.Of(Default9pMsize)
 	expect.Mounts[0].NineP.Cache = ptr.Of(Default9pCacheForRO)
-	expect.Mounts[0].Virtiofs.QueueSize = ptr.Of(DefaultVirtiofsQueueSize)
+	expect.Mounts[0].Virtiofs.QueueSize = nil
 	expect.HostResolver.Hosts = map[string]string{
 		"default": d.HostResolver.Hosts["default"],
 	}
@@ -439,40 +483,46 @@ func TestFillDefault(t *testing.T) {
 	expect.Plain = ptr.Of(false)
 
 	y = LimaYAML{}
-	FillDefault(&y, &d, &LimaYAML{}, filePath)
+	FillDefault(&y, &d, &LimaYAML{}, filePath, false)
 	assert.DeepEqual(t, &y, &expect, opts...)
 
+	dExpect := expect
+
 	// ------------------------------------------------------------------------------------
 	// User-provided defaults should not override user-provided config values
 
 	y = filledDefaults
 	y.DNS = []net.IP{net.ParseIP("8.8.8.8")}
 	y.AdditionalDisks = []Disk{{Name: "overridden"}}
+	y.User.Home = ptr.Of("/root")
 
 	expect = y
 
-	expect.Provision = append(append([]Provision{}, y.Provision...), d.Provision...)
-	expect.Probes = append(append([]Probe{}, y.Probes...), d.Probes...)
-	expect.PortForwards = append(append([]PortForward{}, y.PortForwards...), d.PortForwards...)
-	expect.CopyToHost = append(append([]CopyToHost{}, y.CopyToHost...), d.CopyToHost...)
-	expect.Containerd.Archives = append(append([]File{}, y.Containerd.Archives...), d.Containerd.Archives...)
-	expect.AdditionalDisks = append(append([]Disk{}, y.AdditionalDisks...), d.AdditionalDisks...)
-	expect.Firmware.Images = append(append([]FileWithVMType{}, y.Firmware.Images...), d.Firmware.Images...)
+	expect.Provision = append(append([]Provision{}, y.Provision...), dExpect.Provision...)
+	expect.Probes = append(append([]Probe{}, y.Probes...), dExpect.Probes...)
+	expect.PortForwards = append(append([]PortForward{}, y.PortForwards...), dExpect.PortForwards...)
+	expect.CopyToHost = append(append([]CopyToHost{}, y.CopyToHost...), dExpect.CopyToHost...)
+	expect.Containerd.Archives = append(append([]File{}, y.Containerd.Archives...), dExpect.Containerd.Archives...)
+	expect.Containerd.Archives[2].Arch = *expect.Arch
+	expect.AdditionalDisks = append(append([]Disk{}, y.AdditionalDisks...), dExpect.AdditionalDisks...)
+	expect.Firmware.Images = append(append([]FileWithVMType{}, y.Firmware.Images...), dExpect.Firmware.Images...)
 
 	// Mounts and Networks start with lowest priority first, so higher priority entries can overwrite
-	expect.Mounts = append(append([]Mount{}, d.Mounts...), y.Mounts...)
-	expect.Networks = append(append([]Network{}, d.Networks...), y.Networks...)
+	expect.Mounts = append(append([]Mount{}, dExpect.Mounts...), y.Mounts...)
+	expect.Networks = append(append([]Network{}, dExpect.Networks...), y.Networks...)
 
-	expect.HostResolver.Hosts["default"] = d.HostResolver.Hosts["default"]
+	expect.HostResolver.Hosts["default"] = dExpect.HostResolver.Hosts["default"]
 
-	// d.DNS will be ignored, and not appended to y.DNS
+	// dExpect.DNS will be ignored, and not appended to y.DNS
 
-	// "TWO" does not exist in filledDefaults.Env, so is set from d.Env
-	expect.Env["TWO"] = d.Env["TWO"]
+	// "TWO" does not exist in filledDefaults.Env, so is set from dExpect.Env
+	expect.Env["TWO"] = dExpect.Env["TWO"]
 
-	expect.Param["TWO"] = d.Param["TWO"]
+	expect.Param["TWO"] = dExpect.Param["TWO"]
 
-	FillDefault(&y, &d, &LimaYAML{}, filePath)
+	t.Logf("d.vmType=%q, y.vmType=%q, expect.vmType=%q", *d.VMType, *y.VMType, *expect.VMType)
+
+	FillDefault(&y, &d, &LimaYAML{}, filePath, false)
 	assert.DeepEqual(t, &y, &expect, opts...)
 
 	// ------------------------------------------------------------------------------------
@@ -574,6 +624,7 @@ func TestFillDefault(t *testing.T) {
 				Lima:       "shared",
 				MACAddress: "10:20:30:40:50:60",
 				Interface:  "def1",
+				Metric:     ptr.Of(uint32(25)),
 			},
 			{
 				Lima:      "bridged",
@@ -590,7 +641,7 @@ func TestFillDefault(t *testing.T) {
 			HostIP:         IPv4loopback1,
 			HostPort:       8080,
 			HostPortRange:  [2]int{8080, 8080},
-			Proto:          TCP,
+			Proto:          ProtoTCP,
 		}},
 		CopyToHost: []CopyToHost{{}},
 		Env: map[string]string{
@@ -608,25 +659,33 @@ func TestFillDefault(t *testing.T) {
 			Enabled: ptr.Of(false),
 			BinFmt:  ptr.Of(false),
 		},
+		NestedVirtualization: ptr.Of(false),
+		User: User{
+			Name:    ptr.Of("foo"),
+			Comment: ptr.Of("foo bar baz"),
+			Home:    ptr.Of("/override"),
+			UID:     ptr.Of(uint32(1122)),
+		},
 	}
 
 	y = filledDefaults
 
 	expect = o
 
-	expect.Provision = append(append(o.Provision, y.Provision...), d.Provision...)
-	expect.Probes = append(append(o.Probes, y.Probes...), d.Probes...)
-	expect.PortForwards = append(append(o.PortForwards, y.PortForwards...), d.PortForwards...)
-	expect.CopyToHost = append(append(o.CopyToHost, y.CopyToHost...), d.CopyToHost...)
-	expect.Containerd.Archives = append(append(o.Containerd.Archives, y.Containerd.Archives...), d.Containerd.Archives...)
-	expect.AdditionalDisks = append(append(o.AdditionalDisks, y.AdditionalDisks...), d.AdditionalDisks...)
-	expect.Firmware.Images = append(append(o.Firmware.Images, y.Firmware.Images...), d.Firmware.Images...)
+	expect.Provision = append(append(o.Provision, y.Provision...), dExpect.Provision...)
+	expect.Probes = append(append(o.Probes, y.Probes...), dExpect.Probes...)
+	expect.PortForwards = append(append(o.PortForwards, y.PortForwards...), dExpect.PortForwards...)
+	expect.CopyToHost = append(append(o.CopyToHost, y.CopyToHost...), dExpect.CopyToHost...)
+	expect.Containerd.Archives = append(append(o.Containerd.Archives, y.Containerd.Archives...), dExpect.Containerd.Archives...)
+	expect.Containerd.Archives[3].Arch = *expect.Arch
+	expect.AdditionalDisks = append(append(o.AdditionalDisks, y.AdditionalDisks...), dExpect.AdditionalDisks...)
+	expect.Firmware.Images = append(append(o.Firmware.Images, y.Firmware.Images...), dExpect.Firmware.Images...)
 
-	expect.HostResolver.Hosts["default"] = d.HostResolver.Hosts["default"]
-	expect.HostResolver.Hosts["MY.Host"] = d.HostResolver.Hosts["host.lima.internal"]
+	expect.HostResolver.Hosts["default"] = dExpect.HostResolver.Hosts["default"]
+	expect.HostResolver.Hosts["MY.Host"] = dExpect.HostResolver.Hosts["host.lima.internal"]
 
-	// o.Mounts just makes d.Mounts[0] writable because the Location matches
-	expect.Mounts = append(append([]Mount{}, d.Mounts...), y.Mounts...)
+	// o.Mounts just makes dExpect.Mounts[0] writable because the Location matches
+	expect.Mounts = append(append([]Mount{}, dExpect.Mounts...), y.Mounts...)
 	expect.Mounts[0].Writable = ptr.Of(true)
 	expect.Mounts[0].SSHFS.Cache = ptr.Of(false)
 	expect.Mounts[0].SSHFS.FollowSymlinks = ptr.Of(true)
@@ -639,12 +698,12 @@ func TestFillDefault(t *testing.T) {
 	expect.MountType = ptr.Of(NINEP)
 	expect.MountInotify = ptr.Of(true)
 
-	// o.Networks[1] is overriding the d.Networks[0].Lima entry for the "def0" interface
-	expect.Networks = append(append(d.Networks, y.Networks...), o.Networks[0])
+	// o.Networks[1] is overriding the dExpect.Networks[0].Lima entry for the "def0" interface
+	expect.Networks = append(append(dExpect.Networks, y.Networks...), o.Networks[0])
 	expect.Networks[0].Lima = o.Networks[1].Lima
 
 	// Only highest prio DNS are retained
-	expect.DNS = o.DNS
+	expect.DNS = slices.Clone(o.DNS)
 
 	// ONE remains from filledDefaults.Env; the rest are set from o
 	expect.Env["ONE"] = y.Env["ONE"]
@@ -663,6 +722,13 @@ func TestFillDefault(t *testing.T) {
 	}
 	expect.Plain = ptr.Of(false)
 
-	FillDefault(&y, &d, &o, filePath)
+	expect.NestedVirtualization = ptr.Of(false)
+
+	FillDefault(&y, &d, &o, filePath, false)
 	assert.DeepEqual(t, &y, &expect, opts...)
 }
+
+func TestContainerdDefault(t *testing.T) {
+	archives := defaultContainerdArchives()
+	assert.Assert(t, len(archives) > 0)
+}
