@@ -131,9 +131,6 @@ func (l *LimaQemuDriver) Start(ctx context.Context) (chan error, error) {
 	}
 
 	for i, vhostCmd := range vhostCmds {
-		i := i
-		vhostCmd := vhostCmd
-
 		logrus.Debugf("vhostCmd[%d].Args: %v", i, vhostCmd.Args)
 		if err := vhostCmd.Start(); err != nil {
 			return nil, err
@@ -420,9 +417,9 @@ func (a *qArgTemplateApplier) applyTemplate(qArg string) (string, error) {
 				if !ok {
 					return "", fmt.Errorf("non-string argument %+v", v)
 				}
-				addr := &net.UnixAddr{
-					Net:  "unix",
-					Name: s,
+				addr, err := net.ResolveUnixAddr("unix", s)
+				if err != nil {
+					return "", err
 				}
 				conn, err := net.DialUnix("unix", nil, addr)
 				if err != nil {
