@@ -6,15 +6,14 @@ import (
 	"testing"
 
 	"github.com/lima-vm/lima/pkg/store/filenames"
+	"gotest.tools/v3/assert"
 )
 
 func FuzzLoadYAMLByFilePath(f *testing.F) {
 	f.Fuzz(func(t *testing.T, fileContents []byte) {
 		localFile := filepath.Join(t.TempDir(), "yaml_file.yml")
 		err := os.WriteFile(localFile, fileContents, 0o600)
-		if err != nil {
-			t.Fatal(err)
-		}
+		assert.NilError(t, err)
 		_, _ = LoadYAMLByFilePath(localFile)
 	})
 }
@@ -24,19 +23,13 @@ func FuzzInspect(f *testing.F) {
 		limaDir := t.TempDir()
 		t.Setenv("LIMA_HOME", limaDir)
 		err := os.MkdirAll(filepath.Join(limaDir, "fuzz-instance"), 0o700)
-		if err != nil {
-			t.Fatal(err)
-		}
+		assert.NilError(t, err)
 		ymlFile := filepath.Join(limaDir, "fuzz-instance", filenames.LimaYAML)
 		limaVersionFile := filepath.Join(limaDir, filenames.LimaVersion)
 		err = os.WriteFile(ymlFile, yml, 0o600)
-		if err != nil {
-			t.Fatal(err)
-		}
+		assert.NilError(t, err)
 		err = os.WriteFile(limaVersionFile, limaVersion, 0o600)
-		if err != nil {
-			t.Fatal(err)
-		}
+		assert.NilError(t, err)
 		_, _ = Inspect("fuzz-instance")
 	})
 }
